#pragma once
#include <assert.h>
#include <string>
using namespace std;

class eObj
{
public:
	string name;
	eObj() { }
	eObj(string str) { name=str; }
	eObj(const char *str) { name=str; }
};

class eNode: public eObj
{
public:
	double value;
	bool defined;
	static const double high, low;
	eNode(): value(0), defined(false) {  }
	eNode(string str): value(0), defined(false), eObj(str) { }
	double &v() { return value; }
	bool isDefined() { return defined; }
	bool isHigh() { return value<low; }
	bool isLow() { return value>high; }
	char getLogic() {
		if (!defined) return 'X';
		if (isLow()) return '0';
		else if (isHigh()) return '1';
		else return 'X';
	}
	void set(double v) {
		defined = true;
		value = v;
	}
};

class eDevice: public eObj
{
public:
	string *nodes;
	int *ndx;
	double x, y;
	double orientation;
	double value;
	eDevice() { };
	eDevice(int n);
	virtual ~eDevice();
	virtual int getNodeCount() const = 0; 
	virtual string getType() const = 0;
	virtual string codeString() const = 0;
	virtual int eval(eNode *nodes[]) { printf("inside eDevice\n"); return 0; }
//	virtual void draw(CDC &dc) const = 0;
	void parse(string tok[], int ntokens);
	void parse_value(string tok);
};


class Resistor: public eDevice {
public:
	Resistor(): eDevice(2) { }
	int getNodeCount() const { return 2; }
	string getType() const { return "resistor"; }
	static const string code;
	string codeString() const { return code; }
//	void draw(CDC &dc) const;
};

class Capacitor: public eDevice {
public:
	Capacitor(): eDevice(2) { }
	int getNodeCount() const { return 2; }
	string getType() const { return "capacitor"; }
	static const string code;
	string codeString() const { return code; }
//	void draw(CDC &dc) const;
};

class Inductor: public eDevice {
public:
	Inductor(): eDevice(2) { }
	int getNodeCount() const { return 2; }
	string getType() const { return "inductor"; }
	static const string code;
	string codeString() const { return code; }
//	void draw(CDC &dc) const;
};

class CurrentSource: public eDevice {
public:
	CurrentSource(): eDevice(2) { }
	int getNodeCount() const { return 2; }
	string getType()const { return "current source"; }
	static const string code;
	string codeString() const { return code; }
//	void draw(CDC &dc) const;
};



class eCircuit: public eObj
{
public:
	eNode **nodes;
	int maxnodes, ncount;
	eDevice **devices;
	int maxdevs, ndevices;
	eCircuit() { create(20,8); }
	~eCircuit();
	void create(int node_count, int device_count);
	void clear();
	void read(FILE *in);
	void list_nodes();
	void list_devices();
	void build_node_list();
	eDevice *doAnalog(string type);
	eDevice *doDigital(string type);
	int add_device(eDevice *pdev);
	int add_node(string str);
	eNode *find_node(string str);
	void sysbuild();
//	void draw(CDC &dc);

	// logic simulation
	int cycles;
	//void reset();
	void reset_nodes();

	 int eval();
	 int run();
};

